@@ -15,6 +15,9 @@ class Configuration:
     def bool(self, group, item, default='n'):
         return self.get(group, item, default).lower() in ['y', 'yes', '1', 'true', 'on']
 
+    def list(self, group, item, default='', sep=',', strip=True):
+        return [e.strip() if strip else e for e in self.get(group, item, default).split(sep)]
+
     def number(self, group, item, default='0'):
         value = self.get(group, item, default)
         if value.isnumeric():
